<?php

namespace Drupal\select_number_widget\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\NumberWidget;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'select number' widget.
 *
 * @FieldWidget(
 *   id = "select_number_widget",
 *   label = @Translation("Select number widget"),
 *   field_types = {
 *     "integer",
 *     "decimal",
 *     "float",
 *   }
 * )
 */
class SelectNumberWidget extends NumberWidget {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    switch ($this->fieldDefinition->getType()) {
      case 'integer':
        $form['step'] = [
          '#type' => 'number',
          '#title' => $this->t('Step'),
          '#description' => $this->t('The step value for the number selection.'),
          '#default_value' => $this->getSetting('step') ?? 1,
          '#min' => 1,
        ];
        break;

      case 'decimal':
      case 'float':
        $form['step'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Step'),
          '#description' => $this->t('The step value for the number selection. Supports decimal values.'),
          '#default_value' => $this->getSetting('step') ?? 0.25,
        ];
        break;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'step' => 1,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $step = $this->getSetting('step');
    $summary[] = $this->t('Step: @step', ['@step' => $step]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(
    FieldItemListInterface $items,
    $delta,
    array $element,
    array &$form,
    FormStateInterface $form_state,
  ) {
    $value = $items[$delta]->value ?? 0;
    $field_settings = $this->getFieldSettings();
    if (
      is_numeric($field_settings['min'])
      && is_numeric($field_settings['max'])
    ) {
      // Set minimum and maximum.
      $min = $field_settings['min'];
      $max = $field_settings['max'];
      $step = $this->getSetting('step');

      // Add prefix and suffix.
      if ($field_settings['prefix']) {
        $prefixes = explode('|', $field_settings['prefix']);
        $prefix = Html::decodeEntities(strip_tags(array_pop($prefixes)));
      }
      else {
        $prefix = NULL;
      }
      if ($field_settings['suffix']) {
        $suffixes = explode('|', $field_settings['suffix']);
        $suffix = Html::decodeEntities(strip_tags(array_pop($suffixes)));
      }
      else {
        $suffix = NULL;
      }

      $element += [
        '#type' => 'select',
        '#options' => SelectNumberWidget::selectOptions(
          $min, $max, $step, $prefix, $suffix),
        '#default_value' => $value,
      ];

      return ['value' => $element];
    }
    else {
      return parent::formElement($items, $delta, $element, $form, $form_state);
    }
  }

  /**
   * Get select number widget options.
   */
  public static function selectOptions($min, $max, $step, $prefix, $suffix): array {
    $options = [];

    $pfx = $prefix ?? '';
    $sfx = $suffix ?? '';

    for ($i = $min; $i <= $max; $i += $step) {
      $options[(string) $i] = $pfx . $i . $sfx;
    }

    return $options;
  }

}
